import { Context } from "../../types"
import groupAgent from "./groupAgent"

const ACTION_MAP = {
  sendFunctionSelector: groupAgent.sendFunctionSelector,
  sendTimeScopeSelector: groupAgent.sendTimeScopeSelector,
  manageGroupMsg: groupAgent.manageGroupMsg,
}

/**
 * 处理点击事件
 * @param {Context.Data} ctx - 上下文数据，包含body, larkService和logger
 */
const manageAction = async (ctx: Context.Data) => {
  const { body, logger } = ctx
  const { action } = body?.action?.value as {
    action: keyof typeof ACTION_MAP
  }
  logger.info(`Got lark action: ${action}`)
  if (!action) return
  const func = ACTION_MAP[action]
  if (!func) return
  return func(ctx)
}

/**
 * 处理Action消息
 * @param {Context.Data} ctx - 上下文数据
 */
export const manageActionMsg = async (ctx: Context.Data) => {
  const {
    larkBody: { actionType },
  } = ctx
  if (!["button", "select_static"].includes(actionType!))
    return ctx.genResp.ok()
  const card = await manageAction(ctx)
  if (card) return ctx.genResp.json(card)
  return ctx.genResp.ok()
}
