import logger from "@egg/logger"
import pLimit from "p-limit"

import db from "../db"
import { LarkService } from "../services"

export const resetAccessToken = async () => {
  try {
    const appList = await db.appInfo.getFullList()
    const limit = pLimit(3)
    const service = new LarkService("", "schedule")
    const promiseList = appList.map((app) =>
      limit(() =>
        service.auth.getAk(app.app_id, app.app_secret).then((res) => {
          if (res.code !== 0) return
          return db.tenantAccessToken.update(
            app.id,
            app.name,
            res.tenant_access_token
          )
        })
      )
    )
    await Promise.allSettled(promiseList)
  } catch (error: any) {
    logger
      .child({ requestId: "schedule" })
      .error(`resetAccessToken error: ${error.message}`)
  }
}
